/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package de.schlichtherle.xml;

import java.beans.XMLDecoder;
import java.security.*;
import java.util.Locale;
import javax.annotation.CheckForNull;
import javax.annotation.Nullable;
import net.java.truelicense.core.auth.*;
import net.java.truelicense.core.codec.Codec;
import net.java.truelicense.core.util.Objects;
import static net.java.truelicense.core.util.Objects.*;
import net.java.truelicense.core.util.Strings;
import net.java.truelicense.obfuscate.Obfuscate;
import net.java.truelicense.xml.XmlCodec;

/**
 * Provides interoperability with version 1 (V1) format license keys as used
 * by TrueLicense 1.X applications.
 *
 * @author Christian Schlichtherle
 */
public final class GenericCertificate implements Repository {

    @Obfuscate private static final String BASE64 = "base64";
    @Obfuscate private static final String SIGNATURE_TYPE = "application/octet-stream";
    @Obfuscate private static final String SIGNATURE_TRANSFER_ENCODING = BASE64;
    @Obfuscate private static final String SIGNATURE_ENCODING = "US-ASCII/Base64";

    private final GenericRepository repository;

    private @CheckForNull String encoded, signatureAlgorithm, signatureEncoding,
            signature;

    /** @deprecated This constructor is reserved for {@link XMLDecoder}. */
    @Deprecated
    public GenericCertificate() { this(new XmlCodec()); }

    /**
     * Constructs a generic certificate.
     *
     * @param codec the codec for the artifact.
     */
    public GenericCertificate(final XmlCodec codec) {
        this.repository = new GenericRepository(codec);
    }

    public @Nullable String getEncoded()   { return encoded; }

    public void setEncoded(final String encoded) {
        this.encoded = requireNonNull(encoded);
    }

    public @Nullable String getSignatureAlgorithm()  { return signatureAlgorithm; }

    public void setSignatureAlgorithm(final String signatureAlgorithm) {
        this.signatureAlgorithm = requireNonNull(signatureAlgorithm);
    }

    public @Nullable String getSignatureEncoding() { return signatureEncoding; }

    public void setSignatureEncoding(final String signatureEncoding) {
        this.signatureEncoding = requireNonNull(signatureEncoding);
    }

    public @Nullable String getSignature() { return signature; }

    public void setSignature(final String signature) {
        this.signature = requireNonNull(signature);
    }

    @Override
    public <A> Artifactory<A> sign(
            final Signature engine,
            final PrivateKey key,
            final A artifact)
    throws Exception {
        final GenericRepository r = this.repository;
        final Artifactory<A> a = r.sign(engine, key, artifact);
        setEncoded(r.getArtifactBody());
        setSignatureAlgorithm(engine.getAlgorithm());
        setSignatureEncoding(SIGNATURE_ENCODING);
        setSignature(r.getSignatureBody());
        return a;
    }

    @Override
    public Artifactory<?> verify(
            final Signature engine,
            final PublicKey key)
    throws Exception {
        if (!getSignatureAlgorithm().equalsIgnoreCase(engine.getAlgorithm()))
            throw new IllegalStateException();
        if (!getSignatureEncoding().equalsIgnoreCase(SIGNATURE_ENCODING))
            throw new IllegalStateException();
        final GenericRepository r = this.repository;
        final Codec c = r.codec();
        r.setArtifactType(c.contentType());
        r.setArtifactTransferEncoding(c.contentTransferEncoding());
        r.setArtifactBody(getEncoded());
        r.setSignatureType(SIGNATURE_TYPE);
        r.setSignatureTransferEncoding(SIGNATURE_TRANSFER_ENCODING);
        r.setSignatureBody(getSignature());
        return r.verify(engine, key);
    }

    @Override
    @SuppressWarnings("AccessingNonPublicFieldOfAnotherObject")
    public boolean equals(final Object obj) {
        if (this == obj) return true;
        if (!(obj instanceof GenericCertificate)) return false;
        final GenericCertificate that = (GenericCertificate) obj;
        return Objects.equals(this.encoded, that.encoded)
                && Strings.equalsIgnoreCase(this.signatureAlgorithm, that.signatureAlgorithm)
                && Strings.equalsIgnoreCase(this.signatureEncoding, that.signatureEncoding)
                && Objects.equals(this.signature, that.signature);
    }

    @Override
    public int hashCode() {
        int c = 17;
        c = 31 * c + Objects.hashCode(encoded);
        c = 31 * c + Objects.hashCode(Strings.toLowerCase(signatureAlgorithm, Locale.ROOT));
        c = 31 * c + Objects.hashCode(Strings.toLowerCase(signatureEncoding, Locale.ROOT));
        c = 31 * c + Objects.hashCode(signature);
        return c;
    }
}
